#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sndfile.h>
#include <stdlib.h>



int
do_check (int drop, int argc, char * argv [])
{
  int margin_of_error;
  SNDFILE* goldenWavFile;
  SNDFILE* outputWavFile;
  SF_INFO goldenInfo ;
  SF_INFO outputInfo ;
  short int buffA;
  short int buffB;

  memset (&goldenInfo, 0, sizeof (goldenInfo)) ;
  memset (&outputInfo, 0, sizeof (outputInfo)) ;

  margin_of_error = atoi(argv[1]);

  printf("Checking for Margin of Error %d\n", margin_of_error);

  goldenWavFile = sf_open(argv[2], SFM_READ, &goldenInfo);
  if (goldenWavFile == NULL){      
    printf ("\nERROR : failed Not able to open wav file named '%s' : %s/\n", argv[2], sf_strerror (NULL)) ;
    exit (1) ;
  }

  outputWavFile = sf_open(argv[3], SFM_READ, &outputInfo);
  if (outputWavFile == NULL){      
    printf ("\nERROR : failed Not able to open wav file named '%s' : %s/\n", argv[3], sf_strerror (NULL)) ;
    exit (1) ;
  }
 
  int max_diff = 0;

  // it is possible that the modified pipeline might introduce a few
  // dummy tokens at the beginning of the stream, we'll just drop them
  for(int i = 0; i < drop; i++){
    sf_read_short(outputWavFile, &buffB, 1);
  }

  int count=0;

  while(sf_read_short(goldenWavFile, &buffA, 1)==1){

    if(sf_read_short(outputWavFile, &buffB, 1) != 1){
      // the streams are of different length
      printf("failed (length)\n");
      exit(0);
    }
    
    int td =  (buffA > buffB) ? (buffA - buffB) : (buffB - buffA);
    max_diff = (td > max_diff) ? td : max_diff;
    
    if(max_diff > margin_of_error){
      printf("max_diff %x on drop %d (count %d)\n", max_diff, drop,count);
      return 1;
    }

    count++;
  }

  printf("max_diff %x on drop %d\n", max_diff, drop);

  if(max_diff > margin_of_error)
    return 1;
  else
    return 0;
}

int
main(int argc, char* argv[])
{
  for(int drop = 0; drop < 10; drop++){
    if(do_check(drop, argc, argv)==0){
      printf("passed on drop %d\n", drop);
      exit(0);
    }
  }
  printf("failed\n");
  exit(1);
}
